from datetime import datetime, date, timedelta, timezone

class timedeltacal:
    def __init__(self, **kwargs):
        if "t0" in kwargs:
            # two timestamps
            # result must satisfy t0 + self == t1

            # XXX - Very naive implementation.
            # Very likely wrong, and even if it is correct, 
            # we would probably want all the fields to have the same sign.
            t0 = kwargs["t0"]
            t1 = kwargs["t1"]
            self.years = t1.year - t0.year
            self.months = t1.month - t0.month
            self.days = t1.day - t0.day
            self.hours = t1.hour - t0.hour
            self.minutes = t1.minute - t0.minute
            self.seconds = t1.second - t0.second
            self.microseconds = t1.microsecond - t0.microsecond
            # Adjust
            t2 = t0 + self
            if t2 != t1:
                self.hours -= t2.hour - t1.hour

        else:
            self.years = kwargs.get("years", 0)
            self.months = kwargs.get("months", 0)
            self.days = kwargs.get("weeks", 0) * 7 + kwargs.get("days", 0)
            self.hours = kwargs.get("hours", 0)
            self.minutes = kwargs.get("minutes", 0)
            self.seconds = kwargs.get("seconds", 0)
            self.microseconds = kwargs.get("milliseconds", 0) * 1000 + kwargs.get("microseconds", 0)

    def __radd__(self, t):
        if isinstance(t, datetime):
            # Add years and months.
            m = t.month + self.years * 12 + self.months
            y = t.year + m // 12
            m = m % 12

            # cap days.
            d = t.day
            if m in (4, 6, 9, 11) and t.day > 30:
                d = 30
            elif m == 2:
                if y % 4 == 0 and (y % 100 != 0 or y % 400 == 0):
                    if d > 29:
                        d = 29
                else:
                    if d > 28:
                        d = 28

            # Then add days

            dt = date(y, m, d) + timedelta(days=self.days)

            # And finally add hours and lesser units. We want one hour to
            # always be 3600 seconds, so we convert to UTC and back
            dt1 = t.replace(year=dt.year, month=dt.month, day=dt.day)
            dt2 = dt1.astimezone(timezone.utc)
            dt3 = dt2 + timedelta(hours=self.hours, minutes=self.minutes, seconds=self.seconds, microseconds=self.microseconds)
            dt4 = dt3.astimezone(t.tzinfo)

            return dt4
        elif isinstance(t, date):
            return date(
                    t.year + self.years, t.month + self.months, t.day + self.days,
                    t.tzinfo)
        else:
            raise TypeError()

    def __repr__(self):
        parts = [
            f"{p}={getattr(self, p)}"
                for p in ("years", "months", "days", "hours", "minutes", "seconds", "microseconds")
                    if getattr(self, p)
        ]
        return "timedeltacal(" + ", ".join(parts) + ")"
